<?php
/**
 * An interface for "node-helper" classes
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 * @todo		We need to find a better name for this interface
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
interface NodeHelper extends Helper, AddableCriteria {
	/**
	 * Method to "bootstrap" the node. This step does also apply provided
	 * command-line arguments stored in the request instance. You should now
	 * get it from calling $this->getRequestInstance().
	 *
	 * @return	void
	 */
	function doBootstrapping ();

	/**
	 * Outputs the console teaser. This should only be executed on startup or
	 * full restarts. This method generates some space around the teaser.
	 *
	 * @return	void
	 */
	function outputConsoleTeaser ();

	/**
	 * Add some node-specific filters
	 *
	 * @return	void
	 */
	function addExtraNodeFilters ();

	/**
	 * Generic method to acquire a hub-id. On first run this generates a new one
	 * based on many pseudo-random data. On any later run, unless the id
	 * got not removed from database, it will be restored from the database.
	 *
	 * @param	$requestInstance	A Requestable class
	 * @param	$responseInstance	A Responseable class
	 * @return	void
	 */
	function bootstrapAcquireNodeId (Requestable $requestInstance, Responseable $responseInstance);

	/**
	 * Activates the hub by doing some final preparation and setting
	 * $hubIsActive to TRUE.
	 *
	 * @param	$requestInstance	A Requestable class
	 * @param	$responseInstance	A Responseable class
	 * @return	void
	 */
	function activateNode (Requestable $requestInstance, Responseable $responseInstance);

	/**
	 * Generates a session id which will be sent to the other hubs and peers
	 *
	 * @return	void
	 */
	function bootstrapGenerateSessionId ();

	/**
	 * Generates a private key for en-/decryption
	 *
	 * @return	void
	 */
	function bootstrapGeneratePrivateKey ();

	/**
	 * Initializes the listener pool (class)
	 *
	 * @return	void
	 */
	function initializeListenerPool ();

	/**
	 * Announces this hub to the upper (bootstrap or list) hubs. After this is
	 * successfully done the given task is unregistered from the handler.
	 *
	 * @param	$taskInstance	The task instance running this announcement
	 * @return	void
	 * @throws	NodeAlreadyAnnouncedException	If this hub is already announced
	 */
	function announceToUpperNodes (Taskable $taskInstance);

	/**
	 * Does a self-connect attempt on the public IP address. This should make
	 * it sure, we are reachable from outside world.
	 *
	 * @param	$taskInstance	The task instance running this announcement
	 * @return	void
	 */
	function doSelfConnection (Taskable $taskInstance);

	/**
	 * Determines an instance of a LocateableNode class
	 *
	 * @return	$unlInstance	An instance of a LocateableNode class for this node
	 */
	function determineUniversalNodeLocator ();

	/**
	 * "Getter for an array of an instance of a LocateableNode class
	 *
	 * @return	$unlData	An array of an instance of a LocateableNode class
	 */
	function getUniversalNodeLocatorArray ();

	/**
	 * Updates/refreshes node data (e.g. state).
	 *
	 * @return	void
	 */
	function updateNodeData ();

	/**
	 * Checks whether this node accepts announcements
	 *
	 * @return	$acceptAnnouncements	Whether this node accepts announcements
	 */
	function isAcceptingAnnouncements ();

	/**
	 * Checks whether this node has attempted to announce itself
	 *
	 * @return	$hasAnnounced	Whether this node has attempted to announce itself
	 */
	function ifNodeIsAnnouncing ();

	/**
	 * Checks whether this node has attempted to announce itself and completed it
	 *
	 * @return	$hasAnnouncementCompleted	Whether this node has attempted to announce itself and completed it
	 */
	function ifNodeHasAnnouncementCompleted ();

	/**
	 * Checks wether this node is accepting node-list requests
	 *
	 * @return	$acceptsRequest		Wether this node accepts node-list requests
	 */
	function isAcceptingNodeListRequests ();

	/**
	 * Handles message answer by given data array
	 *
	 * @param	$messageData		A valid answer message data array
	 * @param	$packageInstance	An instance of a Receivable class
	 * @return	void
	 */
	function handleAnswerStatusByMessageData (array $messageData, Receivable $packageInstance);

	/**
	 * "Getter" for an array of all accepted object types
	 *
	 * @return	$objectList		Array of all accepted object types
	 */
	function getListFromAcceptedObjectTypes ();

	/**
	 * Adds extra tasks to the given handler for this node
	 *
	 * @param	$handlerInstance	An instance of a HandleableTask class
	 * @return	void
	 */
	function addExtraTasks (HandleableTask $handlerInstance);
}

// [EOF]
?>
