<?php
/**
 * A NodeMessageRequestNodeList handler
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class NodeMessageRequestNodeListHandler extends BaseMessageHandler implements HandleableMessage, Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set handler name
		$this->setHandlerName('message_request_node_list');

		// Init message data array
		$this->messageDataElements = array(
			XmlRequestNodeListTemplateEngine::REQUEST_DATA_ACCEPTED_OBJECT_TYPES,
		);

		// Init config-copy array
		$this->configCopy = array(
		);

		// Init search data array
		$this->searchData = array(
			XmlRequestNodeListTemplateEngine::REQUEST_DATA_SESSION_ID,
		);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$handlerInstance	An instance of a HandleableMessage class
	 */
	public final static function createNodeMessageRequestNodeListHandler () {
		// Get new instance
		$handlerInstance = new NodeMessageRequestNodeListHandler();

		// Return the prepared instance
		return $handlerInstance;
	}

	/**
	 * Handles data array of the message
	 *
	 * @param	$messageData		An array with message data to handle
	 * @param	$packageInstance	An instance of a Receivable class
	 * @return	void
	 * @throws	RequestNotAcceptedException		If this node does not accept this request
	 */
	public function handleMessageData (array $messageData, Receivable $packageInstance) {
		// Get node instance
		$nodeInstance = NodeObjectFactory::createNodeInstance();

		// Is this node accepting announcements?
		if (!$nodeInstance->isAcceptingNodeListRequests()) {
			/*
			 * This node is not accepting node list requests. Throw an
			 * exception to abort any further processing.
			 */
			throw new RequestNotAcceptedException(array($this, $nodeInstance, $messageData), self::EXCEPTION_REQUEST_NOT_ACCEPTED);
		} // END - if

		// Register the announcing node with this node
		$this->registerNodeByMessageData($messageData);

		// Prepare answer message to be delivered back to the other node
		$this->prepareAnswerMessage($messageData, $packageInstance);
	}

	/**
	 * Adds all required elements from given array into data set instance
	 *
	 * @param	$dataSetInstance	An instance of a StoreableCriteria class
	 * @param	$messageData		An array with all message data
	 * @return	void
	 */
	public function addArrayToDataSet (StoreableCriteria $dataSetInstance, array $messageData) {
		// Add generic first
		parent::addArrayToDataSet($dataSetInstance, $messageData);

		// Add all ements
		foreach ($this->messageDataElements as $key) {
			// Is it there?
			assert(isset($messageData[$key]));

			// Add it
			$dataSetInstance->addCriteria($key, $messageData[$key]);
		} // END - foreach
	}

	/**
	 * Initializes configuration data from given message data array
	 *
	 * @param	$messageData	An array with all message data
	 * @return	void
	 */
	protected function initMessageConfigurationData (array $messageData) {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('REQUEST-HANDLER[' . __METHOD__ . ':' . __LINE__ . ']: messageData=' . print_r($messageData, TRUE));

		// "Walk" throught the config-copy array
		foreach ($this->configCopy as $targetKey => $sourceKey) {
			// Debug message
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('REQUEST-HANDLER[' . __METHOD__ . ':' . __LINE__ . ']: Copying from sourceKey=' . $sourceKey . ' to targetKey=' . $targetKey . '...');

			// Copy from source to targetKey
			$this->getConfigInstance()->setConfigEntry($targetKey, $this->getConfigInstance()->getConfigEntry($sourceKey));
		} // END - foreach

		// Query local DHT for nodes except given session id
		$nodeList = $this->getDhtInstance()->queryLocalNodeListExceptByMessageData(
			$messageData,
			$this,
			XmlRequestNodeListTemplateEngine::REQUEST_DATA_SESSION_ID,
			XmlRequestNodeListTemplateEngine::REQUEST_DATA_ACCEPTED_OBJECT_TYPES,
			BaseHubNode::OBJECT_LIST_SEPARATOR
		);

		// Debug message
		/* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('REQUEST-HANDLER[' . __METHOD__ . ':' . __LINE__ . ']: Got a node list of ' . count($nodeList) . ' entry/-ies back.');

		// Set it serialized in configuration (temporarily)
		$this->getConfigInstance()->setConfigEntry('node_list', base64_encode(json_encode($nodeList)));

		// Translate last exception into a status code
		$statusCode = $this->getTranslatedStatusFromLastException();

		// Set it in configuration (temporarily)
		$this->getConfigInstance()->setConfigEntry(NodeDistributedHashTableDatabaseWrapper::DB_COLUMN_ANSWER_STATUS, $statusCode);
	}

	/**
	 * Removes configuration data with given message data array from global
	 * configuration
	 *
	 * @param	$messageData	An array with all message data
	 * @return	void
	 */
	protected function removeMessageConfigurationData (array $messageData) {
		// "Walk" throught the config-copy array again
		foreach ($this->configCopy as $configKey => $dummy) {
			// Now unset this configuration entry (to save some memory again)
			$this->getConfigInstance()->unsetConfigEntry($configKey);
		} // END - foreach

		// Remove answer status/node list as well
		$this->getConfigInstance()->unsetConfigEntry(NodeDistributedHashTableDatabaseWrapper::DB_COLUMN_ANSWER_STATUS);
		$this->getConfigInstance()->unsetConfigEntry('node_list');
	}
}

// [EOF]
?>
